package com.xrefactory.jedit;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.text.*;
import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.textarea.*;


public class CompletionDialog extends JDialog implements KeyListener {

	DispatchData				data;
	JScrollPane 				scrollPane;
	JTextArea 					textArea;
	String						text;
	LinkedList	 				history;
	String						insertFile;
	int							insertOffset;
	
	static CompletionDialog 	lastcDialog;   // to ensure only one dialog at the time

	static class HistoryElem {
		int 	caret;
		String 	selection;
		HistoryElem(int caret, String selection) {
			this.caret = caret;
			this.selection = selection;
		}
	}

	public static class FqtCompletionDialog extends JDialog {
		static class FqtCompletionPanel extends JPanel implements KeyListener {
			JList 	list;
			String 	pack;
			String 	cl;
			Buffer 	buffer;
			int 	offset;

			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					s.getParentDialog(this).setVisible(false);
				} else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					s.getParentDialog(this).setVisible(false);
					switch (list.getSelectedIndex()) {
					case 0:
						addImportAndInsertCompletion(pack+".*;", buffer, offset, cl);
						break;
					case 1:
						addImportAndInsertCompletion(pack+"."+cl+";", buffer, offset, cl);
						break;
					case 2:
						insertCompletion(buffer, offset, pack+"."+cl);
						break;
					}
				}
			}
			public void keyReleased(KeyEvent e) {e.consume();}
			public void keyTyped(KeyEvent e) {e.consume();}

			FqtCompletionPanel(String pack, String cl, Buffer buffer, int offset) {
				super();
				int y;
				this.pack = pack;
				this.cl = cl;
				this.buffer = buffer;
				this.offset = offset;
				setLayout(new GridBagLayout());
				list = new JList(new String[] {
					"import "+pack+".* and complete "+cl,
					"import "+pack+"."+cl+" and complete "+cl,
					"complete "+pack+"."+cl
				});
				list.addKeyListener(this);
				list.setBackground(s.LIGHT_GRAY);
				list.setSelectedIndex(0);

				JTextArea label = new JTextArea("Fully qualified type name completion will:");
				label.setEditable(false);
				label.setBackground(s.LIGHT_GRAY);
				if (s.javaVersion.compareTo("1.4.0") >= 0) label.setFocusable(false);

				y=0;
				s.addGbcComponent(this, 0,y, 1,1, 1,1, 
								  GridBagConstraints.BOTH,
								  label);
				y++;
				s.addGbcComponent(this, 0,y, 1,1, 1,1, 
								  GridBagConstraints.BOTH,
								  list);
			}
		}

		FqtCompletionDialog(String pack, String cl, Buffer buffer, int offset) {
			super(s.view,"",true);
			Component 	cc;
			enableEvents(AWTEvent.KEY_EVENT_MASK);
			setContentPane(new FqtCompletionPanel(pack, cl, buffer, offset));
			setLocation(s.recommendedLocation(s.getTextArea()));
			s.moveOnScreen(this);
			if (s.javaVersion.compareTo("1.4.0") >= 0) setUndecorated(true);
			pack();
			setVisible(true);
		}
	}

	public static void insertFqtCompletion(Buffer buffer, int offset, String ss) {
		int lastdot = -1;
		int ii;
		while ((ii=ss.indexOf('.', lastdot+1)) != -1) lastdot = ii;
		if (lastdot < 0) {
			insertCompletion(buffer, offset, ss);
			return;
		}
		String pack = ss.substring(0, lastdot);
		String cl = ss.substring(lastdot+1);
		new FqtCompletionDialog(pack, cl, buffer, offset);
	}

	public static void addImportAndInsertCompletion(String imp, Buffer buffer, int offset, String cl) {
		// TODO, do this better
		try {
			DispatchData ndata = new DispatchData(s.xbTask, s.view);
			XrefCharBuffer receipt = ndata.xTask.callProcessOnFileNoSaves(
				new String[]{"-olcxtrivialprecheck", "-getlastimportline"},
				ndata);
			Dispatch.dispatch(receipt,ndata);
			int ln = Integer.parseInt(ndata.info);
			int off = buffer.getLineStartOffset(ln);
			String iinsertion = "import "+imp+"\n";
			buffer.insert(off , iinsertion);
			insertCompletion(buffer, offset+iinsertion.length(), cl);
		} catch (Exception e) {	}
	}

	public static void insertCompletion(Buffer buffer, int offset, String ss) {
		int b, i, blen;
		b = i = offset;
		blen = buffer.getLength();
		while (i<blen && Character.isJavaLetter(buffer.getText(i,1).charAt(0))) i++;
		int cidlen = i-b;
		String cid = buffer.getText(b, i-b);
		String cprefix = s.commonPrefix(cid, ss);
		int cprefixlen = cprefix.length();
		buffer.remove(b+cprefixlen, cidlen-cprefixlen);
		buffer.insert(b+cprefixlen, ss.substring(cprefixlen));
		s.getTextArea().setCaretPosition(offset+ss.length());
	}

	static public String getPrefixOnCompletion() {
		int i,j;
		int offset = s.getCaretPosition();
		i = offset - 1;
		while (i>0 && Character.isJavaLetter(s.getBuffer().getText(i,1).charAt(0))) i--;
		i++;
		return(s.getBuffer().getText(i,offset-i));
	}

	public static int charCount(String ss, char cc, int b, int e) {
		int i = b;
		int res = 0;
		while ((i=ss.indexOf(cc, i))!=-1 && i<e) {
			res ++;
			i ++;
		}
		return(res);
	}

	int getCompletionLineNumber(int car) {
		int line = charCount(text, '\n', 0, car+1);
		return(line);
	}

	void displayProfileIfShouldBeDisplayed() {
		try {
			int i;
			int car = textArea.getCaretPosition();
			int ln = textArea.getLineOfOffset(car);
			int lb = textArea.getLineStartOffset(ln);
			int le = textArea.getLineEndOffset(ln);
			String line = textArea.getText(lb, le-lb);
			// search left par
			for(i=line.length()-1; i>=0; i--) {
				if (line.charAt(i)=='(' 
					&& i<line.length()-1 
					&& i>0
					&& line.charAt(i-1) != ':'
					&& line.charAt(i+1) != ')') break;
				if (line.charAt(i)==':') break;
			}
			if (i>=0 && line.charAt(i)=='(') {
				while (i>0 && line.charAt(i)!=' ') i--;
				String message = line.substring(i,line.length());
				SwingUtilities.invokeLater(new s.MessageDisplayer(message, false));
				//&s.view.getStatus().setMessage(message);
			}
		} catch (Exception e) {
			//& e.printStackTrace(System.err);
		}
	}

	public void keyPressed(KeyEvent e) {
		char key = e.getKeyChar();
		int code = e.getKeyCode();
		int mod = e.getModifiers();
		//&boolean searchFlag = (code==KeyEvent.VK_S && mod==InputEvent.CTRL_MASK);
		boolean searchFlag = (code==KeyEvent.VK_TAB);
		if (code != KeyEvent.VK_UP && code != KeyEvent.VK_DOWN
			&& code != KeyEvent.VK_KP_UP && code != KeyEvent.VK_KP_DOWN
			&& code != KeyEvent.VK_PAGE_UP && code != KeyEvent.VK_PAGE_DOWN
			&& code != KeyEvent.VK_LEFT && code != KeyEvent.VK_RIGHT			
			&& code != KeyEvent.VK_HOME && code != KeyEvent.VK_END			
			) {
			e.consume();
		}
		if (code == KeyEvent.VK_ENTER) {
			int car = textArea.getCaretPosition();
			int line = getCompletionLineNumber(car);
			setVisible(false);
			displayProfileIfShouldBeDisplayed();
			DispatchData ndata = new DispatchData(data, this);
			XrefCharBuffer receipt = ndata.xTask.callProcessSingleOpt("-olcomplselect" + line, ndata);
			Dispatch.dispatch(receipt,ndata);
		} else if (code == KeyEvent.VK_SPACE 
				   && (e.getModifiers() & InputEvent.CTRL_MASK)!=0) {
			closeDialog();
			s.moveToPosition(s.getParentView(this),insertFile, insertOffset);
			completion(s.view);
		} else if (code == KeyEvent.VK_SPACE) {
			int car = textArea.getCaretPosition();
			int line = getCompletionLineNumber(car);
			DispatchData ndata = new DispatchData(data, this);
			XrefCharBuffer receipt = ndata.xTask.callProcessSingleOpt("-olcxcgoto" + line, ndata);
			Dispatch.dispatch(receipt, ndata);
		} else if (code == KeyEvent.VK_BACK_SPACE) {
			if (history.size() > 1) {
				int oldCaret = ((HistoryElem)history.getFirst()).selection.length();
				history.removeFirst();
				int newCaret = ((HistoryElem)history.getFirst()).selection.length();
				showSelection();
				if (newCaret < oldCaret) {
					s.moveToPosition(s.getParentView(this),insertFile, insertOffset);			
					insertOffset--;
					s.getBuffer().remove(insertOffset, 1);
				}
			}
		} else if (Character.isLetterOrDigit(key) || key=='_' || key=='$' || searchFlag) {
			int car = ((HistoryElem) history.getFirst()).caret;
			String sel = ((HistoryElem) history.getFirst()).selection;
			String nsel = sel;
			if (searchFlag) {
				car ++;
			} else {
				nsel = sel+key;
			}
			int ncar = text.toLowerCase().indexOf("\n" + nsel.toLowerCase(), car);
			if (ncar != -1) {
				history.addFirst(new HistoryElem(ncar, nsel));
				showSelection();
				if (! searchFlag) {
					int cind = ncar + nsel.length();
					String ikey = text.substring(cind, cind+1);
					s.moveToPosition(s.getParentView(this),insertFile, insertOffset);			
					s.getBuffer().insert(insertOffset, ikey);
					insertOffset++;
				}
			}
		} else if ("`~!@#$%^&*()_+|-=\\{}[]:\";'<>?,./".indexOf(key) != -1) {
			closeDialog();
			s.moveToPosition(s.getParentView(this),insertFile, insertOffset);
			s.getBuffer().insert(insertOffset, ""+key);
		} else if (code == KeyEvent.VK_ESCAPE) {
			closeDialog();
			if (mod == InputEvent.SHIFT_MASK || mod == InputEvent.ALT_MASK) {
				s.moveToPosition(s.getParentView(this), insertFile, insertOffset);								
			}
//&		} else if (code == KeyEvent.VK_LEFT) {
//&		} else if (code == KeyEvent.VK_RIGHT) {
		}			
	}
	public void keyReleased(KeyEvent e) {
		e.consume();
	}
	public void keyTyped(KeyEvent e) {
		e.consume();
	}

	void showSelection() {
		int car = ((HistoryElem) history.getFirst()).caret;
		String sel = ((HistoryElem) history.getFirst()).selection;
		textArea.setCaretPosition(car);
		textArea.moveCaretPosition(car+sel.length());
	}

	void closeDialog() {
		setVisible(false);
	}

	public static void completion(org.gjt.sp.jedit.View view) {
		if (s.activeProject!=null) {
			XrefStringArray options = new XrefStringArray();
			options.add("-olcxcomplet");
			options.add("-maxcompls="+Opt.maxCompletions());
			if (Opt.completeFullyQualifiedNames()) {
				options.add("-olfqtnametocompletions");
			}
			if (Opt.completionLinkageCheck()) {
				options.add("-olchecklinkage");
			}
			if (Opt.completionAccessCheck()) {
				options.add("-olcheckaccess");
			}
			DispatchData ndata = new DispatchData(s.xbTask, view);
			XrefCharBuffer receipt = ndata.xTask.callProcessOnFileNoSaves(options.toStringArray(true), ndata);
			Dispatch.dispatch(receipt, ndata);
			// do not display project information to not erase profile info 
			// from completions
			//& s.displayProjectInformation();
		}
	}

	CompletionDialog(String completionList, DispatchData data) {
		super(s.view, "");
		if (s.javaVersion.compareTo("1.4.0") >= 0) setUndecorated(true);
		if (lastcDialog!=null && lastcDialog.isVisible()) lastcDialog.setVisible(false);
		lastcDialog = this;
		this.data = data;
		Component 	cc;
		enableEvents(AWTEvent.KEY_EVENT_MASK);
		String 		prefix = getPrefixOnCompletion();
		text = "\n" + completionList;
		textArea = new JTextArea(completionList);
		textArea.setBackground(new Color(255,240,200));
		textArea.setEditable(true);
		insertOffset = s.getCaretPosition();
		insertFile = s.getFileName();
		scrollPane = new JScrollPane(textArea);
		//sp.setMaximumSize(new Dimension(600, 300));
		setContentPane(scrollPane);
		// todo proportional size
		pack();
		Dimension dim = getSize();
		// 20 is for scroll bars
		if (dim.getHeight() > Opt.completionDialogMaxHeight()) {
			dim.setSize(dim.getWidth()+20, Opt.completionDialogMaxHeight());
		}
		if (dim.getWidth() > Opt.completionDialogMaxWidth()) {
			dim.setSize(Opt.completionDialogMaxWidth(), dim.getHeight()+20);
		}
		setSize(dim);
		setLocation(s.recommendedLocation(s.getTextArea()));
		s.moveOnScreen(this);
		textArea.addKeyListener(this);
		history = new LinkedList();
		history.addFirst(new HistoryElem(0, prefix));
		showSelection();
		setVisible(true);
	}
}
