;;; xrefactory.el - (X)Emacs interface to Xrefactory source browser

;; Copyright (C) 1997-2002 Marian Vittek, Xref-Tech
 
;; This  file  is  part  of  Xrefactory  software,  it  implements  an
;; interface between  xref task and (X)Emacs editors.   It is released
;; under  the terms of  the GNU  General Public  License version  2 as
;; published by  the Free Software  Foundation.  The whole  content of
;; this file  is copyrighted by  Xref-Tech. The file does  not contain
;; any  foreign  code written  by  independent developers.   Xref-Tech
;; reserves all  rights for  any future changes  in this  file license
;; politics.   You should  have received  a  copy of  the GNU  General
;; Public License along  with this program; if not,  write to the Free
;; Software Foundation, Inc., 59 Temple  Place - Suite 330, Boston, MA
;; 02111-1307, USA.

;; The  GNU GPL  license  applies only  on  xref.el and  xrefactory.el
;; files.   Distribution  of  other  parts of  Xrefactory,  especially
;; distribution of  the xref task and  its source code  is governed by
;; Xrefactory License Agreement.

;; This  file   may  invoke   functions  which  were   implemented  by
;; independent developers. All the  credits for those functions belong
;; to  authors, contributors  and maintainers  of  packages containing
;; those  functions.  This  concerns mainly  'compile.el', 'comint.el'
;; and 'browse-url.el' packages.  Xrefactory is not an IDE but we have
;; included "Emacs  IDE" submenu into  Xrefactory's (X)Emacs interface
;; in  order  to  allow  users  of  those  packages  to  profite  from
;; Xrefactory's  project management.   Of course,  our acknowledgement
;; belongs  also to  developers and  maintainers of  Emacs  and XEmacs
;; editors.

(provide 'xrefactory)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; You can edit following setting in order to change default key-bindings

(defun xref-add-bindings-to-keymap (keymap)
  (define-key keymap [(f8)] 'xref-completion)
  (define-key keymap [(control f8)] 'xref-ide-compile-run)
  (define-key keymap [(f7)] 'xref-delete-windows)
  (define-key keymap [(control f7)] 'xref-show-stack-symbol-and-references)
  (define-key keymap [(f6)] 'xref-pushCxrefs)
  (define-key keymap [(control f6)] 'xref-listCxrefs)
  (define-key keymap [(f5)] 'xref-popCxrefs)
  (define-key keymap [(control f5)] 'xref-re-push)
  (define-key keymap [(f4)] 'xref-nextref)
  (define-key keymap [(control f4)] 'xref-this-symbol-next-ref)
  (define-key keymap [(f3)] 'xref-previousref)
  (define-key keymap [(control f3)] 'xref-this-symbol-previous-ref)
  (define-key keymap "\M- " 'xref-completion)
  (define-key keymap "\M-\r" 'xref-delete-windows)
)

(defun xref-add-key-bindings-to-local-keymap (keymap)
  ;; mapping available only for local keymaps
  (define-key keymap "\e\t" 'xref-completion)

  ;; plus standard mappings
  (xref-add-bindings-to-keymap keymap)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; An  invocation  of  the  following  function  adds  Xrefactory  key
;; bindings to  current local keymap.  You can use this  function when
;; defining your own programming modes (for example yacc-mode).

(defun xref-add-key-bindings-to-current-local-keymap ()
  (if (not (current-local-map))
	  (use-local-map (make-sparse-keymap))
	)
  (xref-add-key-bindings-to-local-keymap (current-local-map))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; those variables define allowed  suffixes for languages supported by
;; Xrefactory. Suffix of  a file is checked on  invocation of language
;; specific refactorings. You can add your non-standard suffixes here.
;; NOTE! If you wish to  include files with non-standard suffixes into
;; tag file, you have to put also -csuffixes and -javasuffixes options
;; into .xrefrc file.

(defvar xref-c-allowed-suffixes '(".c" ".h" ".tc" ".th"))
(defvar xref-java-allowed-suffixes '(".java"))
(defvar xref-yacc-allowed-suffixes '(".y"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suffixes  determining  buffers where  information  for undoing  are
;; recorded.  Xrefactory  undo will operate  only on files  with those
;; suffixes

(defvar xref-undo-allowed-suffixes '(".java" ".c" ".h" ".tc" ".th" 
									 ".tcc" ".thh" ".y"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;     Functions  loading and  saving source  files, you  may  need to
;;     redefine them if you are  using some kind of version control on
;;     your project.

(defun xref-find-file (file-name)
  (find-file file-name)
)

(defun xref-save-some-buffers (flag)
  (save-some-buffers flag)
)

(defun xref-write-file (name confirm)
  ;; save buffer under its new name (used only with Java refactorings)
  (write-file name confirm)
)

(defun xref-delete-file (name)
  ;; delete file (used only with Java refactorings)
  (delete-file name)
)

(defun xref-move-directory (old-name new-name)
  ;; used only for Java rename package
  (if (eq xref-platform 'windows)
	  (progn
		;; MS-Windows
		(shell-command (format "xcopy \"%s\" \"%s\" /E /Y /I /C /Q" old-name new-name))
		(if (yes-or-no-p "Files copied, delete original folder? ")
			(progn
			  (if (string-match "-nt" system-configuration)
				  (shell-command (format "rd /S /Q \"%s\"" old-name))
				(shell-command (format "deltree /Y \"%s\"" old-name))
				)
			  )))
	;; unix
	(shell-command (format "mv \"%s\" \"%s\"" old-name new-name))
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Xrefactory configuration  variables setting. Majority  of variables
;; can  be set interactively  using 'Options'  subitem of  'Xref' main
;; menu.  In the following, you  can just uncomment commented lines in
;; order to get alternative setting.

;; by default xref does not ask before any global source modification
(if (not (boundp 'xref-detailed-refactoring-confirmations)) 
	(defvar xref-detailed-refactoring-confirmations nil)
  )
;;(setq xref-detailed-refactoring-confirmations t)

;; by default xrefactory does fast-updates inside refactorings
(if (not (boundp 'xref-fast-updates-inside-refactorings)) 
	(defvar xref-fast-updates-inside-refactorings t)
  )
;;(setq xref-fast-updates-inside-refactorings nil)

;; by default xrefactory does auto update Tag BEFORE each refactoring
(if (not (boundp 'xref-auto-update-tag-file-before-refactorings)) 
	(defvar xref-auto-update-tag-file-before-refactorings t)
  )
;;(setq xref-auto-update-tag-file-before-refactorings nil)

;; by default xref asks before starting to browse a javadoc URL
(if (not (boundp 'xref-ask-before-browse-javadoc)) 
	(defvar xref-ask-before-browse-javadoc t)
  )
;;(setq xref-ask-before-browse-javadoc nil)

;; autoredirect is turn off on windows because of some problems
(if (not (boundp 'xref-browse-url-manual-redirect))
	(if (or (string-match "-nt" system-configuration)
			(string-match "-win" system-configuration))
		(defvar xref-browse-url-manual-redirect t)
	  (defvar xref-browse-url-manual-redirect nil)
	  )
  )
;;(setq xref-browse-url-manual-redirect nil)


;; by  default multibyte  buffer representation  is allowed in order
;; to be general
(if (not (boundp 'xref-allow-multibyte)) 
	(defvar xref-allow-multibyte t)
  )
;;(setq xref-allow-multibyte nil)


;; by default truncation is disallowed in order to see profiles
(if (not (boundp 'xref-buffers-truncate-lines)) 
	(defvar xref-buffers-truncate-lines nil)
  )
;;(setq xref-buffers-truncate-lines t)

;; by default the automatic project selection is on.
(if (not (boundp 'xref-current-project)) 
	(defvar xref-current-project nil)
  )
;;(setq xref-current-project "/the/default/project")


;; by default  the accessibility and linkage checks  are not performed
;; on proposed completions
(if (not (boundp 'xref-java-completion-access-check)) 
	(defvar xref-java-completion-access-check nil)
  )
(if (not (boundp 'xref-java-completion-linkage-check)) 
	(defvar xref-java-completion-linkage-check nil)
  )

;; by default  fully qualified type names are proposed when completing
; Java type name
(if (not (boundp 'xref-java-fqt-name-completion)) 
	(defvar xref-java-fqt-name-completion t)
  )

;; coloring is off, it is slow and does not fit with reverse video.
(if (not (boundp 'xref-coloring)) 
	(defvar xref-coloring nil)
  )
;;(setq xref-coloring t)


;; highlighting on mouse movements is on.
(if (not (boundp 'xref-mouse-highlight)) 
	(defvar xref-mouse-highlight t)
  )
;;(setq xref-mouse-highlight nil)


;; store only last few refactorings to undo
(if (not (boundp 'xref-multifile-undo-deep)) 
	(defvar xref-multifile-undo-deep 50)
  )

;; ambiguity resolution on list-references opens three windows
(if (not (boundp 'xref-three-windows)) 
	(defvar xref-three-windows t)
  )

;; the default variable executed by first xref-ide-run command
(if (not (boundp 'xref-ide-last-run-command)) 
	(defvar xref-ide-last-run-command "run")
)


;; the default variable executed by first xref-ide-compile command
(if (not (boundp 'xref-ide-last-compile-command)) 
	(defvar xref-ide-last-compile-command 'compiledir)
)
;; can be also 'compilefile or 'compileproject

;; the default limit for number of completions
(if (not (boundp 'xref-max-completions)) 
	(defvar xref-max-completions 500)
)

;; shell to interprete multilines compile and run commands
(if (not (boundp 'xref-shell)) 
	(defvar xref-shell "sh")
)

;; generate batch file also when compiling with single line command
(if (not (boundp 'xref-always-batch-file))
	(defvar xref-always-batch-file t)
)

;; single insert string completion closes window
(if (not (boundp 'xref-single-completion-closes-window)) 
	(defvar xref-single-completion-closes-window t)
)

(if (not (boundp 'xref-move-point-back-after-refactoring))
	(defvar xref-move-point-back-after-refactoring nil)
)

;; still under testing, auto-update of Tag file
(if (not (boundp 'xref-auto-update-tag-file-before-push))
	(defvar xref-auto-update-tag-file-before-push nil)
)

(if (not (boundp 'xref-auto-switch-to-completion-window))
	(defvar xref-auto-switch-to-completion-window nil)
)

(if (not (boundp 'xref-close-windows-on-pop))
	(defvar xref-close-windows-on-pop nil)
)

(if (not (boundp 'xref-allow-completion-in-tag-search-prompt))
	(defvar xref-allow-completion-in-tag-search-prompt nil)
)

(if (not (boundp 'xref-browse-url-focus-delay))
	(defvar xref-browse-url-focus-delay .1)
)

(if (not (boundp 'xref-window-minimal-size))
	(defvar xref-window-minimal-size 5)
)


;; default size for list and symbol-resolution window size
(defvar xref-win3-src-win-size (/ (frame-height) 3))
(defvar xref-win3-list-win-size (/ (frame-height) 3))
(defvar xref-win3-sym-res-win-size (/ (frame-height) 3))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Completion coloring, you can choose your colors here !

(if (fboundp 'make-face)
	(progn
	  (if (not (boundp 'xref-list-default-face)) 
		  (make-face 'xref-list-default-face)
		)
	  (if (not (boundp 'xref-list-pilot-face)) 
		  (progn
			(make-face 'xref-list-pilot-face)
			(set-face-foreground 'xref-list-pilot-face "navy")
			))
	  (if (not (boundp 'xref-list-symbol-face)) 
		  (progn
			(make-face 'xref-list-symbol-face)
			(set-face-foreground 'xref-list-symbol-face "navy")
			))
	  (if (not (boundp 'xref-keyword-face)) 
		  (progn
			(make-face 'xref-keyword-face)
			(set-face-foreground 'xref-keyword-face "blue")
			))
	  )
 ;; if do not know how to make faces, set the coloring off
	(setq xref-coloring nil)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar xref-preferable-undo-limit 2500000)
(defvar xref-preferable-undo-strong-limit 3000000)

(if (and (boundp 'undo-limit) (> xref-preferable-undo-limit undo-limit))
	(setq undo-limit xref-preferable-undo-limit)
  )
(if (and (boundp 'undo-strong-limit) 
		 (> xref-preferable-undo-strong-limit undo-strong-limit))
	(setq undo-strong-limit xref-preferable-undo-strong-limit)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                      NOTHING MORE TO SETUP                          ;;
;;                    CUSTOMIZATION PART IS OVER                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Here  follows  some e-lisp  code  which  can't  be autoloaded  from
;; xref.el file


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; following code adds standard xrefactory keybinding for current session

(if (or (not (boundp 'xref-key-binding))
		(eq xref-key-binding 'global))
	(progn
	  (xref-add-bindings-to-keymap global-map)
	  ;; following is always set in the GLOBAL keymap !!
	  ;; if you are using some binding for S-mouse-2, put it in comment
	  (global-set-key [(shift mouse-2)] 'xref-find-file-on-mouse)
	  )

  (if (eq xref-key-binding 'local)
	  (progn
		(if (load "cc-mode" t)
			(progn
			  (if (boundp 'c-mode-map)
				  (xref-add-key-bindings-to-local-keymap c-mode-map)
				(message "[Xrefactory] c-mode-map not found, skipping keymap setting")
				)
			  (if (boundp 'java-mode-map)
				  (xref-add-key-bindings-to-local-keymap java-mode-map)
				(message "[Xrefactory] java-mode-map not found, skipping keymap setting")
				)
			  )
		  (message "[Xrefactory] cc-mode file not found, can not setup local key binding, setting it global.")
		  (xref-add-bindings-to-keymap global-map)
		  )

		;; following is always set in the GLOBAL keymap !!
		;; if you are using some binding for S-mouse-2, put it in comment
		(global-set-key [(shift mouse-2)] 'xref-find-file-on-mouse)
		))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; setting Xrefactory customization group

(if (commandp 'customize)
	(progn
	  (defgroup xrefactory nil
"

Here you can set customizable variables controling appearance and
those reactions of Xrefactory which are not project dependent.

")

	  (defcustom xref-auto-update-tag-file-before-refactorings t
		"If on (non-nil), Xrefactory will update tag file before each refactoring. It is highly recommended to keep this option on."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-fast-updates-inside-refactorings t
		"If on (non-nil), Xrefactory provides only fast update of tag file inside refactorings, i.e. reparsing only modified files. If this option is off, then full update of tag is made reparsing also dependent files. Turning this option off will slow down refactorings, but they will be more secure."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-move-point-back-after-refactoring nil
		"If on (non-nil), after finishing a refactoring the point will be moved back to the position where the refactoring started. This is convenient for moving a suite of fields/methods into another file. In such case it is practical that point is not moved to target position, because you wish to select the next field/method to be moved. Note that target position is kept after a refactoring, so it can be reused without setting."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-detailed-refactoring-confirmations nil
		"If on (non-nil), Xrefactory asks confirmation before any source modification during refactorings. Beginners willing to know what actions refactorings really perform may turn this option on. This option also controls whether the window logging update of tag file is displayed. This may help you to determine causes of possible troubles."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-auto-update-tag-file-before-push nil
		"If on (non-nil), Xrefactory will save all opened buffers and update tag file before pushing references on browser stack. If you work on small project and you have fast computer, then it is convenient to turn this option on."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-browse-url-focus-delay .1
		"This variable specifies how many seconds web browser displaying javadoc will be on top of the screen before Emacs regains the focus. The value is a floating point number. If less than zero, then Emacs will not regain the focus at all."
		:type '(number)
		:group 'xrefactory)

	  (defcustom xref-java-completion-access-check nil
		"If on (non-nil), when completing a Java attribute, Xrefactory will propose only accessible symbols checking protected/public/private attributes."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-java-completion-linkage-check nil
		"If on (non-nil), when completing a Java attribute in a static context, Xrefactory will propose only static fields/methods."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-java-fqt-name-completion t
		"If on (non-nil), then completion of a Java class name proposes also fully qualified names of classes stored in jar archives. For example completion of 'LinkedLi' string proposes 'java.util.LinkedList'."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-single-completion-closes-window t
		"If on (non-nil), then completion inserting a string also closes window (if displayed) with previous completions."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-auto-switch-to-completion-window nil
		"If on (non-nil), then when a completion proposes choices the focus is automatically switched into the window with completions."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-window-minimal-size 5
		"Minimal size of window displaying completions and references. If a positive value is given the window will have size proportional to number of completions, but at least xref-window-minimal-size lines. A negative value forbids resizing, the window will take half of the screen not depending on number of completions."
		:type '(integer)
		:group 'xrefactory)

	  (defcustom xref-max-completions 500
		"Maximum number of proposed completions. Completions exceeding this limit are deleted and replaced by ellipsis (...) ."
		:type '(integer)
		:group 'xrefactory)

	  (defcustom xref-multifile-undo-deep 50
		"This variable says how many refactorings are kept in 'undo list, i.e. how many last refactorings can be undone. Xrefactory undo mechanism is build upon standard (X)Emacs undo. To be able to undo large number of refactorings, you have to set also standard (X)Emacs variables 'undo-limit' and 'undo-strong-limit' to sufficient sizes."
		:type '(integer)
		:group 'xrefactory)

	  (defcustom xref-allow-completion-in-tag-search-prompt nil
		"If on (non-nil), then you can use minibuffer completion when entering strings for \"Search Symbol in Tag File\" functions. Completion will propose appropriate completions from the list of all symbols indexed inside Tag file. It is recommended to disable this option for very large projects because xref task can run out of memory while computing the completion list."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-ask-before-browse-javadoc t
		"If on (non-nil), Xrefactory asks user's confirmation before launching JavaDoc browser when trying to move to a definition of symbol without available source code."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-browse-url-manual-redirect xref-browse-url-manual-redirect
		"If on (non-nil), Xrefactory will generate a file with link to JavaDoc when browsing Javadoc with a method having more than one parameter. User has to click on the link manually in order to move to JavaDoc documentation. This option is introduced due to a problem with auto-redirection in IE."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-buffers-truncate-lines nil
		"If on (non-nil), Xrefactory will truncate lines in buffers containing lists of completions and references. Also default formating of completions will be turned off, so exactly one completion by line will appear. Truncated parts of lines can be viewed by scrolling left and right (shift-left and shift-right keys)."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-allow-multibyte t
		"If on (non-nil), Xrefactory will handle buffers containing multibyte characters (China, Japan, Korea ...). However, this will slow down reaction time."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-coloring nil
		"If on (non-nil), Xrefactory will color symbols in completion and reference list buffers. This may slowdown reaction time for large projects."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-mouse-highlight t
		"If on (non-nil), Xrefactory will set highlighting for mouse movement in completion and reference list buffers."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-three-windows t
		"If on (non-nil), Xrefactory opens three windows when listing references of an ambiguous symbol (a virtual method for example). Windows will contain respectively: the source code of actual reference, the list of selected references and symbol resolution menu (with class subtree)."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-ide-last-compile-command 'compiledir
		"Can be either 'compilefile, 'compiledir or 'compileproject. This variable indicates which of compilations will be invoked by 'Emacs IDE -> Last Compile' command. You can preset it to your preferred default value."
		:type '(symbol)
		:group 'xrefactory)

	  (defcustom xref-always-batch-file t
		"If on (non-nil), Xrefactory will generate and then execute a batch file when executing one of Emacs IDE compile functions. If off (nil), Xrefactory will generate batch file only when the compile command exceeds one line."
		:type '(boolean)
		:group 'xrefactory)

	  (defcustom xref-shell "sh"
		"This option determines which shell is used for interpreting batch files generated for Emacs IDE compile and run commands. This variable applies only on Unix like platforms."
		:type '(string)
		:group 'xrefactory)


))

(defun xref-global-options ()
  (interactive "")
  (customize-group 'xrefactory)
)


(defvar xref-home-directory "")


;;(autoload 'xref-push-all-references-within-method "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)

(autoload 'xref-registration "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-encapsulate-field "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-self-encapsulate-field "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-change-field-access-attribute "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-turn-dynamic-to-static "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-turn-static-to-dynamic "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-move-static-field "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-move-field "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-move-static-method "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-move-class "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-pull-up-method "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-push-down-method "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-pull-up-field "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-push-down-field "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-mark-move-target-place "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
;;(autoload 'xref-list-method-non-fqt-names "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-list-method-class-dependencies "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-list-useless-long-names "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-expand-short-refs-of-method "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-reduce-useless-long-names-of-method "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
;;(autoload 'xref-expand-short-refs-of-class "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
;;(autoload 'xref-reduce-useless-long-names-of-class "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-reduce-useless-long-names "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-search-in-completions "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-search-in-tag-file "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-search-definition-in-tag-file "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-search-in-foreign-tag-file "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-completion "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-pushCxrefs-force-ctree "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-pushCxrefs "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-push-name "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-listCxrefs "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-popCxrefs "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-nextref "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-previousref "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-show-stack-top-symbol "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-top-refs-resolution "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-delete-windows "xref" 
		"Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-rename "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-rename-class "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-rename-package "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-gen-html-documentation "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-update-refs "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-fast-update-refs "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-create-refs "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-help "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-about "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)

(autoload 'xref-project-set-active "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-project-new "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-project-delete "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-project-active "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-project-edit-options "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)

(autoload 'xref-arg-push "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-arg-delete "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-arg-insert "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-arg-shift "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-arg-exchange "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)

(autoload 'xref-extract-region-to-function "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-extract-region-to-function-force-address "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-extract-region-to-macro "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-toggle-next-push-manual "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-undo-last-refactoring "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-re-push "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-this-symbol-previous-ref "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-this-symbol-next-ref "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-show-stack-symbol-and-references "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-list-top-references "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-kill-xref-process "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-class-tree-show "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-compile-file "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-compile-dir "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-compile-project "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-compile "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-previous-error "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run-this "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run1 "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run2 "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run3 "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run4 "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-run5 "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-ide-compile-run "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
(autoload 'xref-get-env "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)

(autoload 'xref-find-file-on-mouse "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)

;;(autoload 'xref-ra-intersection "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)
;;(autoload 'xref-ra-symmetric-diff "xref" "Documentation not yet available, please invoke an Xrefactory function first." t)


;; Now follows Xref menu definitions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                      Emacs

;; IDE menu
(defvar xref-ide-menu (make-sparse-keymap "Xref interface to Emacs IDE functions"))
(fset 'xref-ide-menu (symbol-value 'xref-ide-menu))
(define-key xref-ide-menu [xref-ide-compile-run] '("(Last) Compile & Run" . xref-ide-compile-run))
(define-key xref-ide-menu [xref-ide-sep1] '("--"))
(define-key xref-ide-menu [xref-ide-run] '("(Last) Run" . xref-ide-run))
(define-key xref-ide-menu [xref-ide-runthis] '("Run This" . xref-ide-run-this))
(define-key xref-ide-menu [xref-ide-run5] '("Run5" . xref-ide-run5))
(define-key xref-ide-menu [xref-ide-run4] '("Run4" . xref-ide-run4))
(define-key xref-ide-menu [xref-ide-run3] '("Run3" . xref-ide-run3))
(define-key xref-ide-menu [xref-ide-run2] '("Run2" . xref-ide-run2))
(define-key xref-ide-menu [xref-ide-run1] '("Run1" . xref-ide-run1))
(define-key xref-ide-menu [xref-ide-sep2] '("--"))
(define-key xref-ide-menu [xref-ide-preverror] '("Previous Error" . xref-ide-previous-error))
(define-key xref-ide-menu [xref-ide-nexterror] '("Next Error" . xref-ide-next-error))
(define-key xref-ide-menu [xref-ide-sep3] '("--"))
(define-key xref-ide-menu [xref-ide-cmpl] '("(Last) Compile" . xref-ide-compile))
(define-key xref-ide-menu [xref-ide-cprj] '("Compile Project" . xref-ide-compile-project))
(define-key xref-ide-menu [xref-ide-cdir] '("Compile Directory" . xref-ide-compile-dir))
(define-key xref-ide-menu [xref-ide-cfile] '("Compile File" . xref-ide-compile-file))

;; local browse menu
(defvar xref-lm-menu (make-sparse-keymap "Local Motion"))
(fset 'xref-lm-menu (symbol-value 'xref-lm-menu))
(define-key xref-lm-menu [xref-lm-next] '("Next Reference of This Symbol" . xref-this-symbol-next-ref))
(define-key xref-lm-menu [xref-lm-previous] '("Previous Reference of This Symbol" . xref-this-symbol-previous-ref))

;;(defvar xref-sba-menu (make-sparse-keymap "Stack Reference Arithmetics"))
;;(fset 'xref-sba-menu (symbol-value 'xref-sba-menu))
;;(define-key xref-sba-menu [xref-sba-inter] '("Intersection" . xref-ra-intersection))
;;(define-key xref-sba-menu [xref-sba-symdiff] '("Symetric Difference" . xref-ra-symmetric-diff))

(defvar xref-sb-menu (make-sparse-keymap "Browsing with Symbol Stack"))
(fset 'xref-sb-menu (symbol-value 'xref-sb-menu))
;;(define-key xref-sb-menu [xref-sb-force-dialog] '("Force Manual Resolution for Next Push" . xref-toggle-next-push-manual))
(define-key xref-sb-menu [xref-sb-repush] '("Re-push Poped References" . xref-re-push))
(define-key xref-sb-menu [xref-sb-sep3] '("--"))
(define-key xref-sb-menu [xref-sb-pop-and-ret] '("Pop References and Return" . xref-popCxrefs))
(define-key xref-sb-menu [xref-sb-pop] '("Pop References" . xref-poponly))
(define-key xref-sb-menu [xref-sb-sep2] '("--"))
(define-key xref-sb-menu [xref-sb-current-sym-and-refs] '("Show Symbol and References on Top" . xref-show-stack-symbol-and-references))
(define-key xref-sb-menu [xref-sb-current-sym] '("Show Symbol on Top" . xref-top-refs-resolution))
(define-key xref-sb-menu [xref-sb-list-top] '("Show References on Top" . xref-list-top-references))
(define-key xref-sb-menu [xref-sb-caller] '("Initial Position" . xref-caller-reference))
(define-key xref-sb-menu [xref-sb-current] '("Current Reference" . xref-current-reference))
(define-key xref-sb-menu [xref-sb-def] '("Definition Reference" . xref-definition-reference))
(define-key xref-sb-menu [xref-sb-next] '("Next Reference" . xref-nextref))
(define-key xref-sb-menu [xref-sb-previous] '("Previous Reference" . xref-previousref))
(define-key xref-sb-menu [xref-sb-sep1] '("--"))
(define-key xref-sb-menu [xref-sb-push-name] '("Push Symbol by Name" . xref-push-name))
;;(define-key xref-sb-menu [xref-sb-push-ctree] '("Push References via Manual Resolution" . xref-pushCxrefs-force-ctree))
(define-key xref-sb-menu [xref-sb-push-and-list] '("Push and Show Top References" . xref-listCxrefs))
(define-key xref-sb-menu [xref-sb-push-and-go] '("Push References and Goto Definition" . xref-pushCxrefs))
(define-key xref-sb-menu [xref-sb-push] '("Push References" . xref-push-references))


(defvar xref-arg-menu (make-sparse-keymap "Parameter Manipulations"))
(fset 'xref-arg-menu (symbol-value 'xref-arg-menu))
(define-key xref-arg-menu [xref-arg-exchange] '("Exchange Parameters" . xref-arg-exchange))
(define-key xref-arg-menu [xref-arg-shift] '("Move Parameter" . xref-arg-shift))
(define-key xref-arg-menu [xref-arg-insert] '("Insert Parameter" . xref-arg-insert))
(define-key xref-arg-menu [xref-arg-dalete] '("Delete Parameter" . xref-arg-delete))
(define-key xref-arg-menu [xref-arg-push] '("Manual Parameter Edit" . xref-arg-push))

(defvar xref-extr-menu (make-sparse-keymap "Region Extraction"))
(fset 'xref-extr-menu (symbol-value 'xref-extr-menu))
(define-key xref-extr-menu [xref-extr-macro] '("Macro" . xref-extract-region-to-macro))
(define-key xref-extr-menu [xref-extr-function] '("Function/Method" . xref-extract-region-to-function))

(defvar xref-search-menu (make-sparse-keymap "Search Symbol"))
(fset 'xref-search-menu (symbol-value 'xref-search-menu))
(define-key xref-search-menu [xref-search-ftag] '("Foreign Tag File" . 
xref-search-in-foreign-tag-file))
(define-key xref-search-menu [xref-search-tag] '("Tag File" . 
xref-search-in-tag-file))
(define-key xref-search-menu [xref-search-compl] '("Context Completions" .
xref-search-in-completions))

(defvar xref-project-menu (make-sparse-keymap "Project"))
(fset 'xref-project-menu (symbol-value 'xref-project-menu))
(define-key xref-project-menu [xref-prj-edit] '("Edit Options" . 
xref-project-edit-options))
(define-key xref-project-menu [xref-prj-show-active] '("Show Active" . 
xref-project-active))
(define-key xref-project-menu [xref-prj-set-active] '("Set Active" . 
xref-project-set-active))
(define-key xref-project-menu [xref-prj-del] '("Delete" . 
xref-project-delete))
(define-key xref-project-menu [xref-prj-new] '("New" . 
xref-project-new))

;;(defvar xref-rename-menu (make-sparse-keymap "Project"))
;;(fset 'xref-rename-menu (symbol-value 'xref-rename-menu))
;;(define-key xref-rename-menu [xref-rename-other] '("Rename Symbol" . xref-rename))
;;(define-key xref-rename-menu [xref-rename-class] '("Rename Java Class" . xref-rename-class))


(defvar xref-oo-ref-misc-menu (make-sparse-keymap "OO refactorings misc"))
(fset 'xref-oo-ref-misc-menu (symbol-value 'xref-oo-ref-misc-menu))
(define-key xref-oo-ref-misc-menu [xref-rename-class] '("Rename Class" . xref-rename-class))
(define-key xref-oo-ref-misc-menu [xref-rename-pack] '("Rename (Move) Package" . xref-rename-package))
;;(define-key xref-oo-ref-misc-menu [xref-change-fld-attr] '("Change Field Access Attribute" . xref-change-field-access-attribute))
;;(define-key xref-oo-ref-misc-menu [xref-remove-useless] '("Reduce Type's Long Names" . xref-reduce-useless-long-names))
;;(define-key xref-oo-ref-misc-menu [xref-reduce-cshorts] '("Reduce Long Names in Class" . xref-reduce-useless-long-names-of-class))
;;(define-key xref-oo-ref-misc-menu [xref-expand-cshorts] '("Expand Short Names in Class" . xref-expand-short-refs-of-class))
(define-key xref-oo-ref-misc-menu [xref-reduce-mshorts] '("Reduce Long Names in Class/Method" . xref-reduce-useless-long-names-of-method))
(define-key xref-oo-ref-misc-menu [xref-expand-mshorts] '("Expand Short Names in Class/Method" . xref-expand-short-refs-of-method))
(define-key xref-oo-ref-misc-menu [xref-turn-sd] '("Turn Static Method to Dynamic" . xref-turn-static-to-dynamic))
(define-key xref-oo-ref-misc-menu [xref-turn-ds] '("Turn Dynamic Method to Static" . xref-turn-dynamic-to-static))
(define-key xref-oo-ref-misc-menu [xref-sencaps-field] '("Self Encapsulate Field" . xref-self-encapsulate-field))
(define-key xref-oo-ref-misc-menu [xref-encaps-field] '("Encapsulate Field" . xref-encapsulate-field))


(defvar xref-oo-misc-menu (make-sparse-keymap "OO misc"))
(fset 'xref-oo-misc-menu (symbol-value 'xref-oo-misc-menu))
;;(define-key xref-oo-misc-menu [xref-non-fqt] '("List not FQT References in Method" . xref-list-method-non-fqt-names))
;;(define-key xref-oo-misc-menu [xref-inside-method] '("All Reference Inside Method" . xref-push-all-references-within-method))
(define-key xref-oo-misc-menu [xref-this-dep-long] '("List Method's 'This' Dependencies" . xref-list-method-class-dependencies))
(define-key xref-oo-misc-menu [xref-useless-long] '("List Redundant Long Names" . xref-list-useless-long-names))
(define-key xref-oo-misc-menu [xref-class-tree] '("View Class (Sub)tree" . xref-class-tree-show))

(defvar xref-oo-move-menu (make-sparse-keymap "OO Move"))
(fset 'xref-oo-move-menu (symbol-value 'xref-oo-move-menu))
(define-key xref-oo-move-menu [xref-move-class] '("Move Class" . xref-move-class))
(define-key xref-oo-move-menu [xref-push-down] '("Push Down Method" . xref-push-down-method))
(define-key xref-oo-move-menu [xref-push-down-f] '("Push Down Field" . xref-push-down-field))
(define-key xref-oo-move-menu [xref-pull-up] '("Pull Up Method" . xref-pull-up-method))
(define-key xref-oo-move-menu [xref-pull-up-f] '("Pull Up Field" . xref-pull-up-field))
(define-key xref-oo-move-menu [xref-move-f] '("Move Field" . xref-move-field))
(define-key xref-oo-move-menu [xref-move-stm] '("Move Static Method" . xref-move-static-method))
(define-key xref-oo-move-menu [xref-move-stf] '("Move Static Field" . xref-move-static-field))
(define-key xref-oo-move-menu [separator-oo-move2] '("--"))
(define-key xref-oo-move-menu [xref-set-target] '("Set Target Position" . xref-mark-move-target-place))

(defvar xref-misc-menu (make-sparse-keymap "OO Move"))
(fset 'xref-misc-menu (symbol-value 'xref-misc-menu))
(define-key xref-misc-menu [xref-register] '("Register Your Copy" . xref-registration))
(define-key xref-misc-menu [xref-about] '("About Xref" . xref-about))
(define-key xref-misc-menu [xref-help] '("Xref Help" . xref-help))
(define-key xref-misc-menu [xref-kill] '("Kill Xref Process" . xref-kill-xref-process))


(defvar xref-menu (make-sparse-keymap "Xref"))
(if (commandp 'customize)
	(define-key xref-menu [xref-global-options] '("Options" . xref-global-options))
)
(define-key xref-menu [xref-misc-menu] '("Xref Misc" . xref-misc-menu))
(define-key xref-menu [separator-buffers2] '("--"))
(define-key xref-menu [(f7)] '("Remove Xref Window" . xref-delete-windows))
(define-key xref-menu [(f8)] '("Complete Identifier" . xref-completion))
(define-key xref-menu [xref-search-def] '("Search Definition in Tag File" . xref-search-definition-in-tag-file))
(define-key xref-menu [xref-search-menu] '("Search Symbol in" . xref-search-menu))
(define-key xref-menu [separator-buffers5] '("--"))
(define-key xref-menu [xref-undo] '("Undo Last Refactoring" . xref-undo-last-refactoring))
(define-key xref-menu [xref-oo-ref-misc] '("OO Misc Refactorings" . xref-oo-ref-misc-menu))
(define-key xref-menu [xref-oo-move-menu] '("OO Moving" . xref-oo-move-menu))
(define-key xref-menu [xref-extr-menu] '("Extract Region into a" . xref-extr-menu))
(define-key xref-menu [xref-arg-menu] '("Parameter Manipulations" . xref-arg-menu))
(define-key xref-menu [xref-rename-other] '("Rename Symbol" . xref-rename))
;;(define-key xref-menu [xref-rename-menu] '("Safe Renaming" . xref-rename-menu))
(define-key xref-menu [separator-buffers4] '("--"))
(define-key xref-menu [xref-oo-misc-menu] '("OO Misc Browsing" . xref-oo-misc-menu))
(define-key xref-menu [xref-sb-menu] '("Browsing with Symbol Stack" . xref-sb-menu))
(define-key xref-menu [xref-lm-menu] '("Local Motion" . xref-lm-menu))
(define-key xref-menu [separator-buffers3] '("--"))
(define-key xref-menu [xref-gen-html] '("Generate HTML Documentation" . xref-gen-html-documentation))
(define-key xref-menu [xref-update-refs] '("Full Update of Xref Tag File" . xref-update-refs))
(define-key xref-menu [xref-fast-update-refs] '("Update Xref Tag File" . xref-fast-update-refs))
(define-key xref-menu [xref-create-refs] '("Create Xref Tag File" . xref-create-refs))
(define-key xref-menu [separator-buffers6] '("--"))
(define-key xref-menu [xref-ide-menu] '("Emacs IDE" . xref-ide-menu))
(define-key xref-menu [xref-project-menu] '("Project" . xref-project-menu))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                    XEmacs

(defvar xref-xemacs-ide-menu
  '("XEmacs IDE"
   ["Compile File" xref-ide-compile-file t]
   ["Compile Directory" xref-ide-compile-dir t]
   ["Compile Project" xref-ide-compile-project t]
   ["(Last) Compile" xref-ide-compile t]
	"-----"
   ["Next Error" xref-ide-next-error t]
   ["Previous Error" xref-ide-previous-error t]
	"-----"
   ["Run1" xref-ide-run1 t]
   ["Run2" xref-ide-run2 t]
   ["Run3" xref-ide-run3 t]
   ["Run4" xref-ide-run4 t]
   ["Run5" xref-ide-run5 t]
   ["Run This" xref-ide-run-this t]
   ["(Last) Run" xref-ide-run t]
	"-----"
   ["(Last) Compile & Run" xref-ide-compile-run t]
	) "Xref interface to XEmacs IDE functions" 
)

(defvar xref-xemacs-lm-menu
  '("Local Motion"
   ["Previous Reference of This Symbol" xref-this-symbol-previous-ref t]
   ["Next Reference of This Symbol"		xref-this-symbol-next-ref t]
	) "Xref local motion menu for XEmacs" 
)

(defvar xref-xemacs-sb-menu
  '("Browsing with Symbol Stack"
   ["Push References" xref-push-references t]
   ["Push References and Goto Definition" xref-pushCxrefs t]
   ["Push and Show Top References" xref-listCxrefs t]
;;   ["Push References via Manual Resolution" xref-pushCxrefs-force-ctree t]
   ["Push Symbol by Name" xref-push-name t]
	"-----"
   ["Previous Reference" xref-previousref t]
   ["Next Reference" xref-nextref t]
   ["Definition Reference" xref-definition-reference t]
   ["Current Reference" xref-current-reference t]
   ["Initial Position" xref-caller-reference t]
   ["Show References on Top" xref-list-top-references t]
   ["Show Symbol on Top" xref-top-refs-resolution t]
   ["Show Symbol and References on Top" xref-show-stack-symbol-and-references t]
	"-----"
   ["Pop References" xref-poponly t]
   ["Pop References and Return" xref-popCxrefs t]
	"-----"
   ["Re-push Poped References" xref-re-push t]
;;   ["Force Manual Resolution for Next Push" xref-toggle-next-push-manual t]
	) "Xref Source Browsing menu for XEmacs" 
)

(defvar xref-xemacs-arg-menu
  '("Parameter Manipulations"
   ["Manual Parameter Edit"			xref-arg-push t]
   ["Delete Parameter"			xref-arg-delete t]
   ["Insert Parameter"			xref-arg-insert t]
   ["Move Parameter"			xref-arg-shift t]
   ["Exchange Parameters"		xref-arg-exchange t]
	) "Xref parameter manipulations menu for XEmacs" 
)

(defvar xref-xemacs-extr-menu
  '("Extract Region"
   ["into a Function/Method"   xref-extract-region-to-function t]
   ["into a Macro"			   xref-extract-region-to-macro t]
	) "Xref region extraction menu for XEmacs" 
)

(defvar xref-xemacs-search-menu
  '("Search Symbol"
   ["Search in Context Completions"		xref-search-in-completions t]
   ["Search in Tag File"				xref-search-in-tag-file t]
   ["Search in Foreign Tag File"		xref-search-in-foreign-tag-file t]
	) "Xref search menu for XEmacs" 
)

(defvar xref-xemacs-project-menu
  '("Project"
	["New"		        xref-project-new t]
	["Delete"	        xref-project-delete t]
	["Set Active"		xref-project-set-active t]
	["Show Active"		xref-project-active t]
	["Edit Options"		xref-project-edit-options t]
	) "Xref Project menu for XEmacs" 
)

;;(defvar xref-xemacs-rename-menu
;;  '("Safe Renaming"
;;	["Rename Java Class"		        xref-rename-class t]
;;	["Rename Symbol"    		xref-rename t]
;;	) "Xref Renaming menu for XEmacs" 
;;)

(defvar xref-xemacs-oo-misc-menu
  '("OO Misc Browsing"
   ["View Class (Sub)tree"	            xref-class-tree-show t]
   ["List Redundant Long Names"	        xref-list-useless-long-names t]
   ["List Method's 'This' Dependencies"    xref-list-method-class-dependencies t]
;;   ["List not FQT References in Method" xref-list-method-non-fqt-names t]
	) "Xref Miscellaneous OO Functions Menu for XEmacs" 
)

(defvar xref-xemacs-oo-misc-ref-menu
  '("OO Misc Refactorings"
	["Encapsulate Field"	            xref-encapsulate-field t]
	["Self Encapsulate Field"	        xref-self-encapsulate-field t]
	["Turn Dynamic Method to Static"	xref-turn-dynamic-to-static t]
	["Turn Static Method to Dynamic"	xref-turn-static-to-dynamic t]
	["Expand Short Names in Class/Method"	    xref-expand-short-refs-of-method t]
	["Reduce Long Names in Class/Method"	    xref-reduce-useless-long-names-of-method t]
;;	["Expand Short Names in Class"	    xref-expand-short-refs-of-class t]
;;	["Reduce Long Names in Class"	    xref-reduce-useless-long-names-of-class t]
;;	["Reduce Type's Long Names"	        xref-reduce-useless-long-names t]
;;	["Change Field Access Attribute"	xref-change-field-access-attribute t]
	["Rename (Move) Package"	        xref-rename-package t]
	["Rename Class"		                xref-rename-class t]
	) "Xref Miscellaneous OO Refactorings Menu for XEmacs" 
)

(defvar xref-xemacs-oo-moving-menu
  '("OO Moving"
   ["Set Target Position"	        xref-mark-move-target-place t]
	"--"
   ["Move Static Field"	            xref-move-static-field t]
   ["Move Static Method"            xref-move-static-method t]
   ["Move Field"	                xref-move-field t]
   ["Pull Up Field"                 xref-pull-up-field t]
   ["Pull Up Method"                xref-pull-up-method t]
   ["Push Down Field"               xref-push-down-field t]
   ["Push Down Method"              xref-push-down-method t]
   ["Move Class"          		    xref-move-class t]
	) "Xref Symbol Moving Refactorings Menu for XEmacs" 
)

(defvar xref-xemacs-misc-menu
  '("Xref Misc"
   ["Kill Xref Process"				xref-kill-xref-process t]
   ["About Xref"				    xref-about t]
   ["Xref Help"				    	xref-help t]
   ["Register Your Copy"	    	xref-registration t]
   ) "Xref Miscellaneous Functions"
)

(defvar xref-xemacs-menu
  '("Xref"
	"------"
   ["Create Xref Tag File"			xref-create-refs t]
   ["Update Xref Tag File"	        xref-fast-update-refs t]
   ["Full Update of Xref Tag File"	xref-update-refs t]
   ["Generate HTML Documentation"	xref-gen-html-documentation t]
	"--"
	;; browsing menus will come here
	"-----"
	["Rename Symbol"    		    xref-rename t]
	["Undo Last Refactoring"		xref-undo-last-refactoring t]
	"----"
   ["Search Definition in Tag File"	xref-search-definition-in-tag-file t]
   ["Complete identifier"			xref-completion t]
   ["Remove Xref Window"			xref-delete-windows t]
	"---"
	) "Xref menu for XEmacs" 
)


(defvar xref-get-event)

(if (string-match "XEmacs" emacs-version)
	(progn
	;; XEmacs
		(setq xref-get-event 'next-command-event)
		(if window-system 
			(progn 
			  (set-buffer (get-buffer-create " *dummytogetglobalmap*"))
			  (add-submenu nil xref-xemacs-menu)
			  (add-submenu '("Xref") xref-xemacs-project-menu "------")
			  (add-submenu '("Xref") xref-xemacs-ide-menu "------")
			  (add-submenu '("Xref") xref-xemacs-lm-menu "-----")
			  (add-submenu '("Xref") xref-xemacs-sb-menu "-----")
			  (add-submenu '("Xref") xref-xemacs-oo-misc-menu "-----")
			  ;;(add-submenu '("Xref") xref-xemacs-rename-menu "Undo Last Refactoring")
			  (add-submenu '("Xref") xref-xemacs-arg-menu "Undo Last Refactoring")
			  (add-submenu '("Xref") xref-xemacs-extr-menu "Undo Last Refactoring")
			  (add-submenu '("Xref") xref-xemacs-oo-moving-menu "Undo Last Refactoring")
			  (add-submenu '("Xref") xref-xemacs-oo-misc-ref-menu "Undo Last Refactoring")
			  (add-submenu '("Xref") xref-xemacs-search-menu "Search Definition in Tag File")
			  (add-submenu '("Xref") xref-xemacs-misc-menu nil)
			  (if (commandp 'customize)
				  (add-menu-button '("Xref")
								   ["Options" xref-global-options t]
								   ))
			  (kill-buffer " *dummytogetglobalmap*")
			  )
		  )
		)
  ;; Emacs
  (setq xref-get-event 'read-event)
  (let ((menu (lookup-key global-map [menu-bar])))
	(define-key-after menu [xref] 
	  (cons "Xref" xref-menu)
	  (car (nth (- (length menu) 3) menu))
	  )
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                           Custom menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar xref-custom-menu-symbol-count 1)

(defun xref-emacs-add-menu-item (menu text expression)
  ;; first create a new function with expression as body
  (defalias (intern (format "xref-custom-menu-fun-%d" xref-custom-menu-symbol-count)) 
	(append '(lambda () "Xrefactory menu custom function" (interactive "")) (list expression)))
  
  ;; now put new menu item for this function
  (define-key menu 
	(vector (make-symbol (format "xref-custom-menu-sym-%d" xref-custom-menu-symbol-count)))
	(cons text (intern (format "xref-custom-menu-fun-%d" xref-custom-menu-symbol-count))))

  ;; increment counter
  (setq xref-custom-menu-symbol-count (1+ xref-custom-menu-symbol-count))
)

(defun xref-xemacs-add-menu-item (menu text expression after)
  (add-menu-button menu (vector text expression t) after)
)

(defun xref-add-custom-menu-item (text expression)
"Add an item to Xrefactory  'Custom' submenu.  If the 'Custom' submenu
does not  exist, create it.   TEXT is the  name of the  inserted item.
EXPRESSION is an  expression which will be evaluated  when the item is
selected.  For  example: (xref-add-custom-menu-item \"call java2html\"
'(shell-command  (xref-get-env  \"java2html\")))  will  add  an  'call
java2html' menu  item, which will  execute shell command  specified by
the 'java2html' xref environment variable. This variable should be set
by an '-set java2html <command>' option in your .xrefrc file.
"
  (if (string-match "XEmacs" emacs-version)
	  (progn
		(add-submenu '("Xref") '("Custom") "------")
		(xref-xemacs-add-menu-item '("Xref" "Custom") text expression 
								   "XEmacs IDE")
		)
	(if (not (boundp 'xref-custom-menu))
		(progn
		  (defvar xref-custom-menu (make-sparse-keymap "Xref Custom Menu"))
		  (fset 'xref-custom-menu (symbol-value 'xref-custom-menu))
		  (define-key-after xref-menu [xref-custom-menu] 
			(cons "Custom" xref-custom-menu) 
			(car (nth 3 xref-menu)))
		  ))
	(xref-emacs-add-menu-item xref-custom-menu text expression)
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; examples on how to use 'xref-add-custom-menu-item function

;;(xref-add-custom-menu-item 
;; "Call Java2html" 
;; '(shell-command (xref-get-env "java2html"))
;;)
;;(xref-add-custom-menu-item 
;; "Gen HTML Documentation" 
;; '(xref-call-non-interactive-process xref-current-project "-html" 'assynchronized 
;;									 'newwin "Generating HTML. ")
;;)

