package com.xrefactory.jedit;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.event.*;
import org.gjt.sp.jedit.*;

public class BrowserTopPanel extends JSplitPane {

	public static final String refViews[] = {"Filter 0", "Filter 1", "Filter 2", "Filter 3"};
	public static final String treeViews[] = {"Equal Name", "Equal Profile", "Relevant"};

	DispatchData			data;
	
	public TreePanel 		treePanel;
	public ReferencesPanel 	referencesPanel;

	boolean needToUpdate = false;
	//&boolean saveExcursionDuringUpdate = true;

	public class TreePanel extends JPanel {
		
		DispatchData				data;

		public XrefCtree 			xtree;
		public JComboBox 			treeView;
		public ComboBoxTreeFilter	treeFilter;

		class ButtonAll extends JButton implements ActionListener {
			ButtonAll() {super("All"); addActionListener(this);}
			public void actionPerformed( ActionEvent e) {
				s.getParentTreePanel(this).xtree.selectUnselectAll(true);
			}
		}

		class ButtonNone extends JButton implements ActionListener {
			ButtonNone() {super("None"); addActionListener(this);}
			public void actionPerformed( ActionEvent e) {
				s.getParentTreePanel(this).xtree.selectUnselectAll(false);
			}
		}

		class ButtonDefault extends JButton implements ActionListener {
			ButtonDefault() {super("Default"); addActionListener(this);}
			public void actionPerformed( ActionEvent e) {
			}
		}

		class ComboBoxTreeFilter extends JComboBox implements ActionListener {
			ComboBoxTreeFilter() {super(treeViews);	addActionListener(this);}
			public void actionPerformed( ActionEvent e) {
				Opt.browserTreeFilter = treeFilter.getSelectedIndex();
				DispatchData ndata = new DispatchData(data, this);
				XrefCharBuffer receipt = ndata.xTask.callProcessSingleOpt("-olcxmenufilter="+Opt.browserTreeFilter, ndata);
				Dispatch.dispatch(receipt, ndata);
				referencesPanel.refFilter.setSelectedIndex(Opt.browserRefListFilter);
			}			
		}

		TreePanel(DispatchData data) {
			int i,y;
			this.data = data;
			JButton buttons[] = {new ButtonAll(), 
								 new ButtonNone(), 
								 //&new ButtonDefault(),
			};
			xtree = new XrefCtree(data);
			setLayout(new GridBagLayout());
			treeFilter = new ComboBoxTreeFilter();

			y = 0;

			s.addGbcComponent(this, 0,y, buttons.length,1, 1,1, 
							  GridBagConstraints.HORIZONTAL,
							  treeFilter);

			y++;
			s.addGbcComponent(this, 0,y, buttons.length,1, 100,100, 
							  GridBagConstraints.BOTH,
							  new JScrollPane(xtree));

			y++;
			for(i=0; i<buttons.length; i++) {
				s.addGbcComponent(this, i,y, 1,1, 1,1, 
								  GridBagConstraints.HORIZONTAL,
								  buttons[i]);
			}

		}
	}

	public static class ReferencesPanel extends JPanel {

		DispatchData		data;

		XrefRefListArea 	reflist;
		ComboBoxRefsFilter  refFilter;

		class ComboBoxRefsFilter extends JComboBox implements ActionListener {
			ComboBoxRefsFilter() {
				super(refViews); addActionListener(this);
			}
			public void actionPerformed( ActionEvent e) {
				Opt.browserRefListFilter = refFilter.getSelectedIndex();
				ReferencesPanel.this.update();
			}			
		}
		class ButtonPrevious extends JButton implements ActionListener {
			ButtonPrevious() {super("Previous"); addActionListener(this);}
			public void actionPerformed( ActionEvent e) {
				reflist.previous();
			}
		}
		class ButtonNext extends JButton implements ActionListener {
			ButtonNext() {super("Next"); addActionListener(this);}
			public void actionPerformed( ActionEvent e) {
				reflist.next();
			}
		}

		public void updateSelection() {
			DispatchData ndata = new DispatchData(data, this);
			XrefCharBuffer receipt = ndata.xTask.callProcessSingleOpt("-olcxgetcurrentrefn", ndata);
			Dispatch.dispatch(receipt, ndata);
		}

		public void update() {
			DispatchData ndata = new DispatchData(data, this);
			XrefStringArray options = new XrefStringArray();
			options.add("-olcxfilter=" + Opt.browserRefListFilter);
			// need all files to have references up to date
			ndata.xTask.addFileProcessingOptions(options);
			XrefTask.addCommonOptions(options, ndata);
			XrefCharBuffer receipt = ndata.xTask.callProcess(options, ndata);
			Dispatch.dispatch(receipt, ndata);		
		}

		ReferencesPanel(DispatchData data) {
			int i,y;

			this.data = data;
			reflist = new XrefRefListArea(data);
			refFilter = new ComboBoxRefsFilter();

			JButton buttons[] = {new ButtonPrevious(), 
								 new ButtonNext(), 
			};

			JScrollPane xrefs = new JScrollPane(reflist);
			setLayout(new GridBagLayout());
			y = -1;

			y++;
			s.addGbcComponent(this, 0,y, buttons.length,1, 1,1, 
							  GridBagConstraints.HORIZONTAL,
							  refFilter);

			y++;
			s.addGbcComponent(this, 0,y, buttons.length,1, 100,100, 
							  GridBagConstraints.BOTH,
							  xrefs);

			y++;
			for(i=0; i<buttons.length; i++) {
				s.addGbcComponent(this, i,y, 1,1, 1,1, 
								  GridBagConstraints.HORIZONTAL,
								  buttons[i]);

			}
		}
	}

	public void updateData() {
		// save excursion
		s.Position pos = s.getPosition(s.getParentView(this));
		treePanel.treeFilter.setSelectedIndex(Opt.browserTreeFilter);
		// updating reference list is useless, it is updated
		// with update of tree
		//&referencesPanel.refFilter.setSelectedIndex(Opt.browserRefListFilter);
		needToUpdate = false;
		s.moveToPosition(pos);
	}

	public void needToUpdate() {
		if (s.browserIsDisplayed(this)) {
			updateData();
		} else {
			needToUpdate = true;
		}
	}

	public void paint(Graphics g) {
		if (needToUpdate) needToUpdate();
		super.paint(g);
	}

	BrowserTopPanel(DispatchData data,int split) {
		super(HORIZONTAL_SPLIT, false);
		this.data = data;
		treePanel = new TreePanel(data);
		setLeftComponent(treePanel);
		referencesPanel = new ReferencesPanel(data);
		setRightComponent(referencesPanel);
		// TODO! do this seriously
		int tdp = Opt.browserTreeDividerPosition();
		treePanel.setPreferredSize(new Dimension(tdp,200));
		int rdp = 500 - tdp;
		if (rdp < 100) rdp = 100;
		referencesPanel.setPreferredSize(new Dimension(rdp,200));

	}

}

