package sk.xref.java2html;

import java.io.*;

abstract class XrefExtraProcess implements Runnable {

	private String[] execString;
	private final int XREF_EXTRA_PROCESS_BUFFER_SIZE = 1024;
	Process pp;

	abstract public void handleOutputLine(String ss);

	void handleProcessOutput(InputStream stdi) throws IOException {
		byte[] buf = new byte[XREF_EXTRA_PROCESS_BUFFER_SIZE];
		int n, nl;
		String line = "";
		boolean loop = true;
		while (loop) {
			loop = false;
			n = stdi.read(buf, 0, XREF_EXTRA_PROCESS_BUFFER_SIZE);
			if (n>=0) {
				loop = true;
				line += new String(buf,0,n);
				while ((nl=line.indexOf('\n')) >= 0) {
					handleOutputLine(line.substring(0, nl));
					line = line.substring(nl+1);
				}
			}
		}
		handleOutputLine(line);
	}


	// do not call this directly, supposed to called by Thread.start() from
	// run(string) or run(...)
	public void run() {
		try {
 			Runtime rt = Runtime.getRuntime();
			//&for(int i=0; i<execString.length; i++) {
			//&	System.out.println("argv[" + i + "] == " + execString[i]);
			//&}
			pp  = rt.exec(execString);
			InputStream ii = pp.getInputStream();
			InputStream ee = pp.getErrorStream();
			handleProcessOutput(ii);
			kill();
		} catch (Exception e) {
			handleOutputLine("[error] An exception occurs during execution of extern Xref task");
		}
	}

	public void kill() {
		if (pp!=null) {
			pp.destroy();
			pp = null;
		}
	}

	XrefExtraProcess(String[] execString) {
		this.execString = execString;
	}
	
}
